﻿Imports System.IO
Imports System.Drawing.Imaging
Imports System.Collections.ObjectModel
Imports System.Diagnostics
Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.WebEdition
Imports PdfSharp
Imports PdfSharp.Pdf
Imports System.Net

Partial Class MyMapCtl
    Inherits System.Web.UI.UserControl
    Private Const INVALID_ID As Integer = -1

    Private Const GOOGLELAYERNAME As String = "Google Layer"
    Private Const GOOGLEOVERLAYNAME As String = "Google Overlay"
    Private Const THEMATICLAYERNAME As String = "ThematicLayer"
    Private Const MARKERLAYERNAME As String = "MarkerLayer"

#Region " Private Properties "
    Private Const VWSTATE_SHAPEID As String = "VwStShapeId"
    Private Property ShapeId() As Integer
        Get
            If ViewState(VWSTATE_SHAPEID) IsNot Nothing Then
                Return CType(ViewState(VWSTATE_SHAPEID), Integer)
            Else
                Return INVALID_ID
            End If
        End Get
        Set(ByVal value As Integer)
            ViewState(VWSTATE_SHAPEID) = value
        End Set
    End Property

    Private Const VWSTATE_MAPWIDTH As String = "VwStMapWidth"
    Private Property MapWidth() As Integer
        Get
            If ViewState(VWSTATE_MAPWIDTH) IsNot Nothing Then
                Return CType(ViewState(VWSTATE_MAPWIDTH), Integer)
            Else
                Return INVALID_ID
            End If
        End Get
        Set(ByVal value As Integer)
            ViewState(VWSTATE_MAPWIDTH) = value
        End Set
    End Property

    Private Const VWSTATE_MAPHEIGHT As String = "VwStMapHeight"
    Private Property MapHeight() As Integer
        Get
            If ViewState(VWSTATE_MAPHEIGHT) IsNot Nothing Then
                Return CType(ViewState(VWSTATE_MAPHEIGHT), Integer)
            Else
                Return INVALID_ID
            End If
        End Get
        Set(ByVal value As Integer)
            ViewState(VWSTATE_MAPHEIGHT) = value
        End Set
    End Property

    Private ReadOnly Property ShapePath() As String
        Get
            Dim thePath As String = Server.MapPath("~/App_Data")
            If ConfigurationManager.AppSettings("shapePath") IsNot Nothing Then
                thePath = ConfigurationManager.AppSettings("dirMapPath")
            End If
            If Not thePath.EndsWith("\") Then thePath &= "\"
            Return thePath
        End Get
    End Property

#End Region

    Private Class GridItem

        Private _id As Integer
        Public Property Id() As Integer
            Get
                Return _id
            End Get
            Set(ByVal value As Integer)
                _id = value
            End Set
        End Property

        Private _name As String
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property

        Public Sub New(ByVal id As Integer, ByVal name As String)
            _id = id
            _name = name
        End Sub
    End Class

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        If Not IsPostBack Then
            InitMap()
            ShapeId = 1010
            If ShapeId <> INVALID_ID Then
                AddShape(ShapeId, True, True)
            End If

            Dim items As New List(Of GridItem)
            For i As Integer = 0 To 10
                items.Add(New GridItem(i, "Item " & i))
            Next
            grdTest.DataSource = items
            grdTest.DataBind()
        Else
            ctlMap.CurrentExtent = GetMapBoundingBox()
        End If

    End Sub

    Private Sub InitMap()
        System.Diagnostics.Debug.Print("Map version = " & ctlMap.Version)

        ctlMap.MapBackground.BackgroundBrush = New GeoSolidBrush(GeoColor.FromHtml("#B3C6D4"))
        ctlMap.CurrentExtent = New RectangleShape(-13939426.6371, 6701997.4056, -7812401.86, 2626987.386962)
        ctlMap.MapUnit = GeographyUnit.Meter
        ctlMap.MapTools.Logo.Enabled = False
        ctlMap.MapTools.PanZoomBar.Enabled = True

        'Using Google API string from ThinkGeo samples - we have our own for production
        Dim googleMapKey As String = "http://maps.google.com/maps?file=api&amp;v=2&amp;key=ABQIAAAAoxK_HcqphMsnUQHEwLwHlRSavkNJi0NVTgm4UDidoiIU5dUJpRQW88FufPCp0aTPraxZgZFAIUHn3Q"
        If ConfigurationManager.AppSettings("googleMapKey") IsNot Nothing Then
            googleMapKey = ConfigurationManager.AppSettings("googleMapKey")
        End If

        Dim googleOverlay As New GoogleOverlay(GOOGLEOVERLAYNAME)
        googleOverlay.JavaScriptLibraryUri = New Uri(googleMapKey)
        googleOverlay.GoogleMapType = GoogleMapsMapType.RoadMap
        ctlMap.CustomOverlays.Add(googleOverlay)

        'Create the static Google map layer
        Dim googleMapLayer As New GoogleMapsLayer(GOOGLELAYERNAME)
        googleMapLayer.LicenseKey = googleMapKey
        googleMapLayer.MapType = GoogleMapsMapType.RoadMap
        Dim lo As New LayerOverlay(GOOGLELAYERNAME)
        lo.Layers.Add(googleMapLayer)
        lo.IsVisible = False
        ctlMap.CustomOverlays.Add(lo)
    End Sub

    Public Sub RefreshMap()
        ClearMap()
        AddShape(ShapeId, True, True)
    End Sub

    Private Sub ClearMap()
        For layerIdx As Integer = ctlMap.CustomOverlays.Count - 1 To 0 Step -1
            If ctlMap.CustomOverlays(layerIdx).Name <> THEMATICLAYERNAME AndAlso _
               ctlMap.CustomOverlays(layerIdx).Name <> GOOGLELAYERNAME AndAlso _
               ctlMap.CustomOverlays(layerIdx).Name <> GOOGLEOVERLAYNAME Then
                ctlMap.CustomOverlays.Remove(layerIdx)
            End If
        Next
    End Sub

    Private Sub AddShape(ByVal shapeId As Integer, ByVal highlight As Boolean, ByVal setExtent As Boolean)
        Dim dirLayer As ShapeFileFeatureLayer = Nothing
        For Each shapeLayer As Overlay In ctlMap.CustomOverlays
            If shapeLayer.Name = "D" & shapeId.ToString() Then
                Dim tempLayer As LayerOverlay = DirectCast(shapeLayer, LayerOverlay)
                dirLayer = DirectCast(tempLayer.Layers(0), ShapeFileFeatureLayer)
                Exit For
            End If
        Next

        If dirLayer IsNot Nothing Then
            dirLayer.IsVisible = True
            If highlight Then
                dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(150, GeoColor.SimpleColors.LightGreen)
            Else
                dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(50, GeoColor.SimpleColors.LightOrange)
            End If
            ctlMap.CurrentExtent = GetMapBoundingBox()
        Else
            Dim shapeIdStr As String = String.Format("{0:000000}", shapeId)

            If File.Exists(ShapePath & shapeIdStr & ".shp") Then
                dirLayer = New ShapeFileFeatureLayer(ShapePath & shapeIdStr & ".shp")
                dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.Color = GeoColor.FromArgb(255, GeoColor.SimpleColors.Black)
                If highlight Then
                    dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(150, GeoColor.SimpleColors.LightGreen)
                Else
                    dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(50, GeoColor.SimpleColors.LightOrange)
                End If
                dirLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City1("DIRID")
                dirLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
                dirLayer.Name = "D" & shapeId.ToString()

                Dim proj4 As New Proj4Projection(Proj4Projection.GetEpsgParametersString(4326), Proj4Projection.GetGoogleMapParametersString())
                dirLayer.FeatureSource.Projection = proj4

                Dim shapeOverlay As New LayerOverlay("D" & shapeId.ToString(), False, TileType.SingleTile)
                shapeOverlay.Layers.Add(dirLayer)
                shapeOverlay.TransitionEffect = TransitionEffect.None

                ctlMap.CustomOverlays.Add(shapeOverlay)
                ctlMap.CurrentExtent = GetMapBoundingBox()
            End If
        End If

    End Sub

    Private Function GetMapBoundingBox() As RectangleShape
        Dim result As RectangleShape = Nothing

        For layerIdx As Integer = 0 To ctlMap.CustomOverlays.Count - 1
            If ctlMap.CustomOverlays(layerIdx).Name <> GOOGLELAYERNAME AndAlso _
               ctlMap.CustomOverlays(layerIdx).Name <> GOOGLEOVERLAYNAME AndAlso _
               ctlMap.CustomOverlays(layerIdx).Name <> THEMATICLAYERNAME AndAlso _
               ctlMap.CustomOverlays(layerIdx).Name <> MARKERLAYERNAME Then
                Dim shapeOverlay As LayerOverlay = ctlMap.CustomOverlays(layerIdx)
                Dim l As ShapeFileFeatureLayer = DirectCast(shapeOverlay.Layers(0), ShapeFileFeatureLayer)
                l.FeatureSource.Open()

                Dim feats As System.Collections.ObjectModel.Collection(Of Feature) = l.FeatureSource.GetAllFeatures(New String() {"DIRID"})

                Dim boundingBox As RectangleShape = feats(0).GetBoundingBox()
                If result Is Nothing Then
                    result = boundingBox
                Else
                    result.ExpandToInclude(boundingBox)
                End If

                l.FeatureSource.Close()
            ElseIf ctlMap.CustomOverlays(layerIdx).Name = MARKERLAYERNAME Then
                Dim markerOverlay As InMemoryMarkerOverlay = DirectCast(ctlMap.CustomOverlays(layerIdx), InMemoryMarkerOverlay)
                markerOverlay.FeatureSource.Open()
                Dim feats As System.Collections.ObjectModel.Collection(Of Feature) = markerOverlay.FeatureSource.GetAllFeatures(New String() {"ADDRESS"})

                Dim boundingBox As RectangleShape = feats(0).GetBoundingBox()
                If result Is Nothing Then
                    result = boundingBox
                Else
                    result.ExpandToInclude(boundingBox)
                End If

                markerOverlay.FeatureSource.Close()

            End If
        Next

        If result Is Nothing Then
            result = New RectangleShape(-13939426.6371, 6701997.4056, -7812401.86, 2626987.386962)
        End If

        Return result
    End Function

    Protected Sub grdTest_RowCreated(ByVal sender As Object, ByVal e As System.Web.UI.WebControls.GridViewRowEventArgs) Handles grdTest.RowCreated
        If e.Row.RowType = DataControlRowType.DataRow Then
            Dim chk As CheckBox = e.Row.FindControl("chkSelectDir")
            If chk IsNot Nothing Then
                chk.AutoPostBack = True
                AddHandler chk.CheckedChanged, AddressOf SurroundDirCheckChanged
            End If
        End If
    End Sub

    Protected Sub SurroundDirCheckChanged(ByVal sender As Object, ByVal e As EventArgs)
        Dim chk As CheckBox = DirectCast(sender, CheckBox)
    End Sub

    Protected Sub lnkbExportJpeg_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles lnkbExportJpeg.Click
        Dim tempFileNameGuid As Guid = Guid.NewGuid()
        Dim tempFileName As String = tempFileNameGuid.ToString().Replace("-", "") & ".jpg"
        Dim tempPath As String = "~/temp"
        If Not tempPath.EndsWith(System.IO.Path.AltDirectorySeparatorChar) Then
            tempPath &= System.IO.Path.AltDirectorySeparatorChar
            tempFileName = tempPath & tempFileName
        End If

        ctlMap.CustomOverlays(GOOGLELAYERNAME).IsVisible = True
        ctlMap.ActiveBaseOverlay = ctlMap.CustomOverlays(GOOGLELAYERNAME)
        'ctlMap.CurrentExtent = ExtentHelper.GetDrawingExtent(ctlMap.CurrentExtent, ctlMap.Width.Value, ctlMap.Height.Value)
        ctlMap.CurrentExtent = ExtentHelper.SnapToZoomLevel(ctlMap.CurrentExtent, ctlMap.MapUnit, ctlMap.Width.Value, ctlMap.Height.Value, New ZoomLevelSet())
        ctlMap.CurrentScale = ExtentHelper.GetScale(ctlMap.CurrentExtent, ctlMap.Width.Value, ctlMap.MapUnit)

        Dim bmp As System.Drawing.Bitmap = ctlMap.GetBitmap(ctlMap.Width.Value, ctlMap.Height.Value)

        ' todo: if you are still using Version 3.1.299, please add the code  as following here
        ' DrawInMem(bmp)


        ctlMap.ActiveBaseOverlay = ctlMap.CustomOverlays(GOOGLEOVERLAYNAME)
        ctlMap.CustomOverlays(GOOGLELAYERNAME).IsVisible = False

        Dim jpgEncoder As ImageCodecInfo = GetEncoder(ImageFormat.Jpeg)
        Dim qualityEncoder As System.Drawing.Imaging.Encoder = System.Drawing.Imaging.Encoder.Quality
        Dim jpgEncoderParms As New System.Drawing.Imaging.EncoderParameters(1)
        Dim jpgQualityParm As New System.Drawing.Imaging.EncoderParameter(qualityEncoder, 90)
        jpgEncoderParms.Param(0) = jpgQualityParm
        bmp.Save(Server.MapPath(tempFileName), jpgEncoder, jpgEncoderParms)
        ScriptManager.RegisterStartupScript(Me.Page, GetType(String), "popupTearPageMap", "window.open('" & ResolveUrl(tempFileName) & "');", True)
    End Sub

    Private Sub DrawInMem(ByVal bitmap As System.Drawing.Bitmap)
        If ctlMap.CustomOverlays IsNot Nothing AndAlso ctlMap.CustomOverlays.Count > 0 Then
            For Each overlay As Overlay In ctlMap.CustomOverlays
                Dim markers As New Collection(Of Marker)()
                Dim simpleMarkerOverlay As SimpleMarkerOverlay = TryCast(overlay, SimpleMarkerOverlay)
                If simpleMarkerOverlay IsNot Nothing Then
                    markers = simpleMarkerOverlay.Markers
                End If

                Dim inMemoryMarkerOverlay As InMemoryMarkerOverlay = TryCast(overlay, InMemoryMarkerOverlay)
                If inMemoryMarkerOverlay IsNot Nothing Then
                    markers = inMemoryMarkerOverlay.GetMarkers(ctlMap.CurrentExtent, ctlMap.ClientZoomLevelScales(ctlMap.CurrentScale) + 1)
                End If

                Dim geoCanvas As New GdiPlusGeoCanvas()

                DrawMarkers(bitmap, geoCanvas, markers)
            Next
        End If

    End Sub
    Private Sub DrawMarkers(ByVal bitmap As System.Drawing.Bitmap, ByVal geoCanvas As GdiPlusGeoCanvas, ByVal markers As Collection(Of Marker))
        For Each marker As Marker In markers
            Dim markerImage As System.Drawing.Bitmap = Nothing
            If marker.WebImage.RotationAngle <> 0.0F OrElse Not [String].IsNullOrEmpty(marker.WebImage.Text) Then
                Dim iconPath As String = Page.Request.Url.AbsoluteUri.Substring(0, Page.Request.Url.AbsoluteUri.LastIndexOf("/"c) + 1) + Server.MapPath(marker.WebImage.ImageVirtualPath)
                Dim request As WebRequest = WebRequest.Create(iconPath)
                Dim response As HttpWebResponse = DirectCast(request.GetResponse(), HttpWebResponse)
                markerImage = New System.Drawing.Bitmap(response.GetResponseStream())
                response.Close()
            Else
                markerImage = New System.Drawing.Bitmap(Page.MapPath("~/theme/default/img/marker_blue.gif"))
            End If
            geoCanvas.BeginDrawing(bitmap, ctlMap.CurrentExtent, ctlMap.MapUnit)
            geoCanvas.DrawWorldImageWithoutScaling(markerImage, marker.Position.X, marker.Position.Y, DrawingLevel.LevelOne)
            geoCanvas.EndDrawing()
        Next
    End Sub


    Private Function GetEncoder(ByVal format As ImageFormat) As ImageCodecInfo

        Dim codecs As ImageCodecInfo() = ImageCodecInfo.GetImageDecoders()

        Dim codec As ImageCodecInfo
        For Each codec In codecs
            If codec.FormatID = format.Guid Then
                Return codec
            End If
        Next codec
        Return Nothing

    End Function

    Protected Sub lnkbExportPdf_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles lnkbExportPdf.Click
        Dim currentExtent As RectangleShape = ctlMap.CurrentExtent
        Dim googleLayer As New GoogleMapsLayer("ABQIAAAAoxK_HcqphMsnUQHEwLwHlRSavkNJi0NVTgm4UDidoiIU5dUJpRQW88FufPCp0aTPraxZgZFAIUHn3Q")
        Dim shapeLayer As ShapeFileFeatureLayer = DirectCast(DirectCast(ctlMap.CustomOverlays(GOOGLEOVERLAYNAME), LayerOverlay).Layers("ShapeLayer"), ShapeFileFeatureLayer)

        shapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(100, 212, 220, 184), GeoColor.FromArgb(255, 132, 132, 154), 1)
        shapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

        Dim doc As New PdfDocument()
        Dim page As PdfPage = doc.AddPage()
        page.Size = PageSize.A4

        Dim canvas As New GoogleMapPdfGeoCanvas()

        Dim resolution As Double = Math.Max(ctlMap.CurrentExtent.Width / ctlMap.Width.Value, ctlMap.CurrentExtent.Height / ctlMap.Height.Value)
        Dim newDrawingExtent As New RectangleShape(ctlMap.CurrentExtent.UpperLeftPoint.X, ctlMap.CurrentExtent.UpperLeftPoint.Y, _
                                                   ctlMap.CurrentExtent.UpperLeftPoint.X + ctlMap.Width.Value * resolution, _
                                                   ctlMap.CurrentExtent.UpperLeftPoint.Y - ctlMap.Height.Value * resolution)

        canvas.DrawingArea = New System.Drawing.Rectangle(0, 0, 800, 600)

        canvas.BeginDrawing(page, ctlMap.CurrentExtent, GeographyUnit.Meter)

        System.Diagnostics.Debug.WriteLine("GoogleLayer begin drawing")
        googleLayer.Open()
        googleLayer.Draw(canvas, New Collection(Of SimpleCandidate)())
        googleLayer.Close()
        System.Diagnostics.Debug.WriteLine("GoogleLayer end drawing")

        System.Diagnostics.Debug.WriteLine("ShapeLayer begin drawing")
        shapeLayer.Open()
        shapeLayer.Draw(canvas, New Collection(Of SimpleCandidate)())
        shapeLayer.Close()
        System.Diagnostics.Debug.WriteLine("ShapeLayer end drawing \r\r")

        canvas.EndDrawing()

        doc.Save("c:\\test1.pdf")
        Process.Start("c:\\test1.pdf")

    End Sub

    Private Sub PlotLocation(ByVal street As String, ByVal city As String, ByVal state As String)
        If Not String.IsNullOrEmpty(street.Trim()) AndAlso Not String.IsNullOrEmpty(city.Trim()) AndAlso Not String.IsNullOrEmpty(state.Trim()) Then
            Dim point As GeoCoordinate = GoogleGeocode.GetCoordinates(street, city, state)

            Dim mrkLayer As InMemoryMarkerOverlay
            If Not ctlMap.CustomOverlays.Contains(MARKERLAYERNAME) Then
                mrkLayer = New InMemoryMarkerOverlay(MARKERLAYERNAME)
                mrkLayer.FeatureSource.Open()
                mrkLayer.Columns.Add(New FeatureSourceColumn("ADDRESS"))
                mrkLayer.FeatureSource.Close()
                mrkLayer.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.WebImage = New WebImage("theme/default/img/marker.gif")
                mrkLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

                mrkLayer.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.Popup.ContentHtml = "<span class='popup'>[#ADDRESS#]</span>"

                ' Set the correct projection (Decimal Degrees to Google’s)
                Dim proj4 As New Proj4Projection(Proj4Projection.GetEpsgParametersString(4326), Proj4Projection.GetGoogleMapParametersString())
                mrkLayer.FeatureSource.Projection = proj4

                ctlMap.CustomOverlays.Add(mrkLayer)
                ctlMap.CustomOverlays(MARKERLAYERNAME).IsVisible = True
            Else
                mrkLayer = DirectCast(ctlMap.CustomOverlays(MARKERLAYERNAME), InMemoryMarkerOverlay)
            End If

            mrkLayer.Features.Add("L1", _
                                  New Feature(point.Longitude, point.Latitude, "L1", New String() {"ADDRESS:" & street.Trim() & "<br/>" & city.Trim() & "<br/>" & state.Trim()}))
        End If

    End Sub

    Protected Sub lnkbPlotLocation_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles lnkbPlotLocation.Click
        PlotLocation("1215 Frazier Road", "Albertville", "AL")
    End Sub
End Class
